package org.greenstone.gsdl3.gs3build.extractor;

import java.io.FileReader;

import java.net.URL;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.IndexDocument;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentLoader;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;

public class IndexExtractor implements ExtractorInterface
{
  class IndexHandlerException extends Exception
  { public IndexHandlerException(String value)
    { super(value);
    }
  }

  /**
   *  An inner class to handle GML files
   */
  class IndexHandler
  { String  content;
    String  line;
    int     pos;
    boolean doneRow;
    List    labels;
    URL     base;

    IndexHandler(String content, URL url, DocumentList documentList) throws IndexHandlerException
    { this.content = content;
      this.doneRow = false;
      this.labels  = new ArrayList();
      this.base    = url;

      String parentDir;
      int leaf = this.base.toString().lastIndexOf('/');
      if (leaf >= 0) {
	parentDir = this.base.toString().substring(0, leaf+1);
      }
      else {
	parentDir = this.base.toString();
      }

      // get the first line
      this.getLine();

      if (!this.hasMore())
      { throw new IndexHandlerException("No title line");
      }
      
      // get the first totem - it should be "key:"
      String entry = this.getEntry(true);
      
      // now get all the labels
      while (this.hasMore())
      { String label = this.getEntry(true);
        if (label == null)
	  continue;
	label.trim();

	if (label.length() == 0) {
	  continue;
	}
	
	this.labels.add(label);
	System.out.println("Adding label: " + label);
      }

      while (this.hasMoreLines()) {
	this.getLine();

	// Get the file pattern itself
	String filePattern = this.getEntry(true);
	if (filePattern == null) {
	  continue;
	}

	filePattern.trim();
	if (filePattern.length() == 0) {
	  continue;
	}

	// get a list of documents that match the file pattern
	List files = documentList.findDocumentIdsUsingFile(filePattern);
	if (files != null) {
	  Iterator iterator = files.iterator();
	  while (iterator.hasNext()) {
	    System.out.println("Matches file " + iterator.next().toString());
	  }
	}

	// if no files match this data, then skip this row
	// TODO: raise a quality error message
	if (files == null || files.size() == 0) {
	  continue;
	}

	// TODO: cache up the documents that match for speed?

	// Next, split the row into the separate metadata items
	int entryNo = 0;
	while (this.hasMore()) {
	  String item = this.getEntry(true);
	  if (item == null) {
	    entryNo ++;
	    continue;
	  }

	  item.trim();
	  if (item.length() == 0) {
	    entryNo ++;
	    continue;
	  }

	  String label = null;
	  if (item.startsWith("<")) {
	    int labelEnd = item.indexOf('>');
	    if (labelEnd >= 0) {
	      label = item.substring(1, labelEnd);

	      item = item.substring(labelEnd+1, item.length());

	      // eliminate any weird whitespace
	      item.trim();

	      // cope with a solo 'item' label with no following string
	      if (item.length() == 0) {
		entryNo ++;
		continue;
	      }
	    }
	    // starts with a bracketed label
	  }
	  else if (entryNo < this.labels.size()) {
	    label = (String) this.labels.get(entryNo);
	  }

	  // Actually post the metadata -
	  // it may be good to have cached all the documents that we're going to change
	  // in order to minimise rewrites...
	  if (label != null) {
	    System.out.println("Posting " + label + "=" + item + " to " + parentDir + filePattern);
	    // Post to document
	  }
	  entryNo ++;
	}
      }
    }

    private boolean hasMore()
    { return this.line != null;
    }

    private boolean hasMoreLines()
    { return this.content != null;
    }

    private String getEntry()
    { return this.getEntry(false);
    }

    private String getEntry(boolean breakSpace)
    { String reply;
      int start, tab = 0;
      boolean quoted = false;

      start = 0;
      while (start < this.line.length() &&
	     this.line.charAt(start) == ' ') {
	start ++;
      }

      if (start == this.line.length()) {
	this.line = null;
	return null;
      }

      if (this.line.charAt(start) == '"') {
	quoted = true;
	breakSpace = false;
	start ++;
      }
      tab = start;

      while (tab != this.line.length() &&
	     this.line.charAt(tab) != '\t' &&
	     !(quoted && this.line.charAt(tab) == '"') &&
             !(this.line.charAt(tab) == ' ' && breakSpace))
      { tab ++;
      }

      if (start > 0) {
	this.line = this.line.substring(start);
	tab -= start;
      }

      if (tab == this.line.length()) {
	reply = this.line;
	this.line = null;
      }
      else {
	reply = this.line.substring(0, tab);
	this.line = this.line.substring(tab+1);
      }

      return reply;
    }

    private String getLine()
    { if (this.content == null) {
	this.line = null;
	return null;
      }
	
      do {
        int eol = this.content.indexOf('\n');
	if (eol < 0) {
	  this.line = this.content;
	  this.content = null;
	}
	else {
	  this.line = this.content.substring(0, eol);
	  this.content = this.content.substring(eol+1);
	  while (this.content.length() > 0 &&
		 this.content.charAt(0) < ' ')
	  { this.content = this.content.substring(1);
	  }
	}

	if (this.line != null) {
	  this.line.trim();
	}
      } while (this.content != null && this.line != null && this.line.length() == 0);

      return this.line;
    }
  }

  private DocumentList documentList;

  /**
   *  Construct of extractor
   */
  public IndexExtractor()
  { // Intentionally left blank
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void configure(String outputDir)
  { // Intentionally left blank
  }

  public void configure(DocumentList list)
  { this.documentList = list;
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void startPass(int passNo)
  { // Intentionally left blank
  }

  /**
   *  Process the document - for a GML document, this results in the
   *  decoration of other files, for other documents, it does nothing.
   */
  public void extractDocument(DocumentID docID, DocumentInterface document)
  { if (document.getDocumentType().equals(IndexDocument.INDEX_DOCUMENT_TYPE))
    { // Extract the content from the index file
      
      // get the file
      String documentText = 
	DocumentLoader.getAsString(document.getDocumentFiles().getFile(0).getURL());
	
      if (documentText == null) {
	System.err.println("IndexExtractor: Unable to load any content for " + document.getDocumentFiles().getFile(0).toString());
	return;
      }

      try {
	IndexHandler handler = new IndexHandler(documentText, document.getDocumentFiles().getFile(0).getURL(), this.documentList);
      }
      catch (IndexHandlerException ex) {
      }
      
      // for each document post it to the corresponding document
    }
  }

  protected static void postMetadata(String file, String value, String label)
  {
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void endPass(int passNo)
  { // Intentionally left blank
  }

  /**
   *  This extractor is a simple, single-pass extractor
   *
   *  @see: org.greenstone.gsdl3.gs3build.extractor.ExtractorInterface:getNumberOfPasses
   */
  public int getNumberOfPasses()
  { return 1;
  }
}
